unit Character;

interface

uses
  World, Graphics, Windows;

const
  VelIncrement: Extended = 0.5; //Inkrement za Vx
  SmallMaxVel: Extended = 4; //Max Vx kada hoda
  LargeMaxVel: Extended = 7.5; //Max Vy kada trci
  Friction: Extended = 0.5; //Trenje kada stane sa trcanjem
  SmallJumpImpulse: Extended = 14; //Vertikalni impuls malog skoka
  LargeJumpImpulse: Extended = 16; //Vertikalni impuls velikog skoka
  Gravity: Extended = 0.5; //Gravitacija koja utice kada je [flgFalling] in Flags
type
  TCharacter = class
  private
  public
    World: TWorld;
    //Flegovi (indikatori stanja)
    Flags: set of (flgGoingLeft, flgGoingRight, flgJumping, flgRunning, flgFalling, flgFacingLeft, flgFacingRight);
    X: Extended; //X Koordinata
    Y: Extended; //Y Koordinata
    Width: Integer;  //Sirina igraca, MOZE se bezbedno menjati u toku igre!!!
    Height: Integer; //Visina igraca, MOZE se bezbedno menjati u toku igre (naravno, ako na tren. poziciji ima mesta za novu velicinu)!!!
    PriorX: Extended; //Prethodni (pouzdano "dobri") X
    PriorY: Extended;  //Prethodni (pouzdano "dobri") Y
    Vx: Extended;//X komponenta brzine
    Vy: Extended;//Y komponenta brzine
    Orient: String; //Orijentacija (okrenut levo, okrenut desno, moze biti 'L' ili 'R', trenutno potrebna zbog grafike
                    //Trenutno potrebna glavnom programu, bice izbaceno
                    //Koristice se flgFacingLeft i flgFacingRight
    JumpImpulse: Extended; //Inicajalni skok, moze biti Large ili SmallJumpImpulse
    MaxVel: Extended; //Maksimalna brzina, moze biti Large ili SmallMaxVel
    constructor Create(AWorld: TWorld; AWidth, AHeight, StartX, StartY: Integer);
    procedure Tick; //Otkucaj, ciklus...
    procedure GoLeft; //Idi levo (pritisnut taster)
    procedure GoRight; //Idi desno (pritisnut taster)
    procedure GoJump; //Skoci (pritisnut taster)
    procedure GoRun; //Trci (pritisnut taster)
    procedure StopLeft; //Stani levo (pusten taster)
    procedure StopRight; //Stani desno (pusten taster)
    procedure StopJump; //Stani sa skokom (pusten taster)
    procedure StopRun; //Stani sa trcanjem (pusten taster)
    function GetX: Integer; //Vraca trenutni X
    function GetY: Integer; //Vraca trenutni Y
  end;

implementation

{ TCharacter }

constructor TCharacter.Create(AWorld: TWorld; AWidth, AHeight, StartX, StartY: Integer);
begin
  World := AWorld; //Potrebno zbog World.IsAvailable metoda
  X := StartX; //PocetniX
  Y := StartY; //PocetniY
  Width := AWidth;
  Height := AHeight;
  Vx := 0;
  Vy := 0;
  Flags := [flgFacingRight];
  Orient := 'R'; //bice kasnije izbaceno kompletno Orient polje
  MaxVel := SmallMaxVel;
  JumpImpulse := SmallJumpImpulse;
end;

procedure TCharacter.GoLeft;
begin
  Flags := Flags + [flgGoingLeft];
  Flags := Flags - [flgGoingRight];
  Flags := Flags + [flgFacingLeft];
  Flags := Flags - [flgFacingRight];
  Orient := 'L';
  if Vx > 3 then  //Da ne bi malo "proklizavao" ako je prethodno isao desno
    Vx := 3;
end;

procedure TCharacter.GoRight;
begin
  Flags := Flags - [flgGoingLeft];
  Flags := Flags + [flgGoingRight];
  Flags := Flags - [flgFacingLeft];
  Flags := Flags + [flgFacingRight];
  Orient := 'R';
  if Vx < -3 then //Da ne bi malo "proklizavao" ako je prethodno isao levo
    Vx := -3;
end;

procedure TCharacter.GoJump;
begin
  if not (flgFalling in Flags) then
  begin
    Vy := JumpImpulse;
    Flags := Flags + [flgJumping];
  end;
end;

procedure TCharacter.GoRun;
begin
  Flags := Flags + [flgRunning];
  MaxVel := LargeMaxVel; //Povecaj maksimalnu brzinu posto sad trci
end;

procedure TCharacter.StopJump;
begin
  if flgJumping in Flags then
  begin
    Flags := Flags - [flgJumping];
    if Vy > 6 then  //Da ne bi TRENUTNO stao sa skokom pri pustanju Jump tastera
      Vy := 6;
  end;
end;

procedure TCharacter.StopLeft;
begin
  Flags := Flags - [flgGoingLeft];
end;

procedure TCharacter.StopRight;
begin
  Flags := Flags - [flgGoingRight];
end;

procedure TCharacter.StopRun;
begin
  Flags := Flags - [flgRunning];
  MaxVel := SmallMaxVel; //Smanji maksimalnu brzinu posto sad hoda
end;

procedure TCharacter.Tick;
begin
  //Provera za visinu skoka - ako lik trci onda vise skace
  if (flgRunning in Flags) and (Abs(Vx) > 0) then
    JumpImpulse := LargeJumpImpulse
  else
    JumpImpulse := SmallJumpImpulse;

  //Izracunavanje Vx i X
  if flgGoingRight in Flags then
  begin
    if Vx < MaxVel then
      Vx := Vx + VelIncrement;
    if Vx > MaxVel then
      Vx := MaxVel;
  end;
  if flgGoingLeft in Flags then
  begin
    if Vx > -MaxVel then
      Vx := Vx - VelIncrement;
    if Vx < -MaxVel then
      Vx := -MaxVel;
  end;
  //Ako ne ide ni levo ni desno onda deluje trenje
  if not ((flgGoingLeft in Flags) or (flgGoingRight in Flags)) then
    if Vx <> 0 then
      if Vx > 0 then
        Vx := Vx - Friction
      else
        Vx := Vx + Friction;

  PriorX := X; //Zapamti stari (dobri) X
  X := X + Vx; //Dodaj sracunatu brzinu

  //Proveri i eventualno ispravi novi polozaj
  if not (World.IsAvailablePos(Trunc(X), Trunc(Y)) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y) + Height) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y)) and World.IsAvailablePos(Trunc(X), Trunc(Y) + Height)) then
  begin
  //znaci ne valja X
    repeat
      if X < PriorX then
        X := X + 1
      else
        X := X - 1;
    until (World.IsAvailablePos(Trunc(X), Trunc(Y)) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y) + Height) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y)) and World.IsAvailablePos(Trunc(X), Trunc(Y) + Height));
    Vx := 0; //Postavi Vx na 0 da se ne bi igrac privremeno "zalepio" za objekat ako se zakuco maksimalnom brzinom i treba vremena da deluje trenje
  end;

  //Izracunavanje Vy i Y...
  //Ako je dosao do vrha svog skoka
  if (flgJumping in Flags) and (Vy <= 0) then
  begin
    Flags := Flags - [flgJumping];
    Vy := 0;
  end;
  //Utvrditi da li lik pada, tj. da li je u vazduhu
  if World.IsAvailablePos(Trunc(X), Trunc(Y-1)) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y-1)) then
    Flags := Flags + [flgFalling]
  else
    Flags := Flags - [flgFalling];

  //Ako pada onda deluje gravitacija
  if flgFalling in Flags then
    Vy := Vy - Gravity;
  //Ako ne pada i ne skace
  if not((flgJumping in Flags) or (flgFalling in Flags)) then
  begin
    Vy := 0;
  end;
  PriorY := Y; //Zapamti stari dobri Y
  Y := Y + Vy;

  //Proveri novi Y
  if not (World.IsAvailablePos(Trunc(X), Trunc(Y)) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y) + Height) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y)) and World.IsAvailablePos(Trunc(X), Trunc(Y) + Height)) then
  //znaci ne valja Y
  begin
    repeat
      if Y < PriorY then
        Y := Y + 1
      else
        Y := Y - 1;
    until (World.IsAvailablePos(Trunc(X), Trunc(Y)) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y) + Height) and World.IsAvailablePos(Trunc(X) + Width, Trunc(Y)) and World.IsAvailablePos(Trunc(X), Trunc(Y) + Height));
    Vy := 0; //Isto ko Vx = 0;
  end;
end;

function TCharacter.GetX: Integer;
begin
  Result := Trunc(X);
end;

function TCharacter.GetY: Integer;
begin
  Result := Trunc(Y);
end;


end.
